/*********************************************************************/
/* GEMP.C  - This program times the monostable on the game port.     */
/* A Radio Shack thermistor has been connected to pins 1 & 3.  It    */
/* calibrated, using the program GP1.C.  The calibration data is     */
/* put in the table, and a linear interpolation is done to convert   */
/* the gameport count to temperature.  Every computer/thermistor     */
/* must be calibrated for accurate results.                          */
/*                                                                   */
/* This program was written in Borland's Turbo C.  Some changes may  */
/* be needed if other C compilers are used.                          */
/* July 1994 - Gary C. Sutcliffe  W9XT                               */
/*********************************************************************/


#include <stdio.h>
#include <dos.h>

#define TSIZE  16   /* number of table entries */
main()
{
float table[TSIZE][2] = { 21, 40.3,  23, 37.4,  25, 33,  28, 29.4,  29, 27,
		    32, 24.7, 33, 24.1,  37, 20,  41, 17.4,  45, 15.8,
		    49, 11.8,  54, 9.8,  62, 5.8,  65, 4.3,  70, 2.8,
		    77, 0.0 };  /*This is the count to temperature
		    table.  First number is count, second is temperature.
		    This data would need to be changed for other systems &
		    thermistors. */

int count;          /* game port count value */
int i;              /* general purpose*/
float temp;         /* temperature */
float fcount;       /* count converted to floating point*/
float dc;           /*delta count - used for interpolation*/

count = 0;

disable();  	       /* disable interrupts so clock wont interfere */
outportb(0x201,0);     /* trigger game port monostable */
while(inportb(0x201) & 0x01)count++;  /* time the monostable */
enable();             /* turn interrupts back on */


fcount = (float)count;
temp = 0.0;
if(fcount < table[0][0])temp = 999;  /*out of range - report it*/
if(fcount > table[TSIZE-1][0])temp = -999;  /*out of range*/
if(temp == 0.0)
	{      /* do a linear interpolation */
	i = 1;
	while(fcount >= table[i][0] && i <= TSIZE-1)i++;
	dc = (fcount - table[i-1][0])/(table[i][0] - table[i-1][0]);
	temp = ((table[i-1] - table[i]) * dc) + table[i-1][1];
	}
if(fcount - (float)((int)fcount) >= .05)fcount = fcount + 1;  /*round it off*/
printf("\n\nTemperature = %dC\n",(int)temp);

}  /* end of GTEMP.C */
